﻿/// A type that can be coerced to a byte slice.
pub trait Array {
	/// Coerce a type to a byte slice.
	fn as_slice(&self) -> &[u8];
}
// #[cfg(feature = "unsize")]
// /// Coerce byte arrays and vectors to an unsized slice.
// impl<A: core::array::FixedSizeArray<u8>> Array for A {
// 	fn as_slice(&self) -> &[u8] {
// 		self.as_slice()
// 	}
// }
//#[cfg(all(not(feature = "unsize"), any(feature = "std", feature = "alloc")))]
#[cfg(feature = "alloc")]
use alloc::vec::Vec;
#[cfg(any(feature = "std", feature = "alloc"))]
/// Coercion of byte vectors to unsized slices
impl Array for Vec<u8> {
	fn as_slice(&self) -> &[u8] {
		self.as_slice()
	}
}

// #[cfg(not(feature = "unsize"))]
macro_rules! slices {
  ($v:literal $( $a:literal )*) => {
    impl Array for &[u8; 0 $( + $a )*] {
      fn as_slice(&self) -> &[u8] {
        (*self).as_slice()
      }
    }
    impl Array for [u8; 0 $( + $a )*] {
      fn as_slice(&self) -> &[u8] {
        &*self
      }
    }
    slices!($($a )*);
  };
  () => {
    impl Array for &[u8] {
      fn as_slice(&self) -> &[u8] {
        *self
      }
    }
  };
}
// #[cfg(feature = "unsize")]
// macro_rules! slices {
// 	($($_:literal )*) => {};
// }
slices!(
	1 1 1 1 1 1 1 1
	1 1 1 1 1 1 1 1
	1 1 1 1 1 1 1 1
	1 1 1 1 1 1 1 1
);
